const http = require('http');
const axios = require('axios');
const config = require('config');
const ms = require('ms');
const getLogger = require('../utils/logger/logger');
const {
    JWR_TOKEN_TENANCY_AUDIENCE, JWT_TOKEN_API_AUTH0_AUDIENCE, CLIENT_ID, CLIENT_SECRET, JWT_TOKEN_AUTH0
} = require('../utils/consts');


const logger = getLogger();

let tenancyServiceToken;
let occmServiceToken;


async function createServiceToken(audience) {
    const { data } = await axios.post(`${JWT_TOKEN_AUTH0}/oauth/token`, {
        audience,
        client_id: CLIENT_ID,
        client_secret: CLIENT_SECRET,
        grant_type: 'client_credentials'
    }, {
        timeout: ms(config.get('timeout')),
        httpAgent: new http.Agent({ keepAlive: true })
    });
    return { token: `${data.token_type} ${data.access_token}`, expiresIn: data.expires_in };
}

async function createTenancyServiceToken() {
    logger.info('Get tenancy service token');
    const { token, expiresIn } = await createServiceToken(JWR_TOKEN_TENANCY_AUDIENCE);
    tenancyServiceToken = token;
    logger.info('tenancy token received');
    const shouldBeActivatedIn = ms(`${expiresIn}s`) - ms('30m');
    logger.info(`setTimeout with ${shouldBeActivatedIn} `);
    setTimeout(() => createTenancyServiceToken(), shouldBeActivatedIn);
}

async function createOccmServiceToken() {
    logger.info('Get occm service token');
    const { token, expiresIn } = await createServiceToken(JWT_TOKEN_API_AUTH0_AUDIENCE);
    occmServiceToken = token;
    logger.info('tenancy token received');
    const shouldBeActivatedIn = ms(`${expiresIn}s`) - ms('30m');
    logger.info(`setTimeout with ${shouldBeActivatedIn} `);
    setTimeout(() => createTenancyServiceToken(), shouldBeActivatedIn);
}

function getTenancyServiceToken() {
    return tenancyServiceToken;
}

function getOccmServiceToken() {
    return occmServiceToken;
}

module.exports = {
    createTenancyServiceToken, createOccmServiceToken, getTenancyServiceToken, getOccmServiceToken
};
