const AWS = require('aws-sdk');

const AWS_REGIONS = {
    'us-east-2': 'US East (Ohio)',
    'us-east-1': 'US East (N. Virginia)',
    'us-west-1': 'US West (N. California)',
    'us-west-2': 'US West (Oregon)',
    'ap-south-1': 'Asia Pacific (Mumbai)',
    'ap-northeast-3': 'Asia Pacific (Osaka-Local)',
    'ap-northeast-2': 'Asia Pacific (Seoul)',
    'ap-southeast-1': 'Asia Pacific (Singapore)',
    'ap-southeast-2': 'Asia Pacific (Sydney)',
    'ap-northeast-1': 'Asia Pacific (Tokyo)',
    'ca-central-1': 'Canada (Central)',
    'cn-north-1': 'China (Beijing)',
    'cn-northwest-1': 'China (Ningxia)',
    'eu-central-1': 'EU (Frankfurt)',
    'eu-west-1': 'EU (Ireland)',
    'eu-west-2': 'EU (London)',
    'eu-west-3': 'EU (Paris)',
    'eu-north-1': 'EU (Stockholm)',
    'sa-east-1': 'South America (São Paulo)'
};

async function getAwsRegionByRegionCode(regionCode) {
    const regionName = AWS_REGIONS[regionCode];
    return { region: regionCode, regionName: regionName || regionCode };
}

async function getAwsRegions() {
    const ec2 = new AWS.EC2({ region: 'us-east-1', accessKeyId: process.env.AWS_ACCESS_KEY, secretAccessKey: process.env.AWS_SECRET_KEY });
    const regions = await ec2.describeRegions().promise();
    return regions.Regions.map(region => {
        const { RegionName: regionCode } = region;
        return getAwsRegionByRegionCode(regionCode);
    });
}

module.exports = {
    getAwsRegions, AWS_REGIONS, getAwsRegionByRegionCode
};
