const { AGENTS_MGMT_ENDPOINT, API_PATH_AGENTS, CLOUD_TIERING_NAMESPACE } = require('../../utils/consts');
const { getAgentsSchema, getAgentSchema } = require('../../joi-schemas/agents-schemas');
const { validateResponse } = require('../../utils/schema-validation');
const getLogger = require('../../utils/logger/logger');
const { getTenancyServiceToken } = require('../auth0');
const { getRequest } = require('./cloud-manager-axios-utils');

const logger = getLogger();


async function getAgents(accountId) {
    logger.info(`Get agents of accountId ${accountId}`);
    const data = await getRequest(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${accountId}`,
        {
            'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
            authorization: getTenancyServiceToken()
        });
    validateResponse(data, getAgentsSchema);
    return data;
}

async function getAgent(agentId, accountId) {
    logger.info(`Get agent ${agentId} of accountId ${accountId}`);
    const data = await getRequest(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agentId}?account_id=${accountId}`, {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    });
    validateResponse(data, getAgentSchema);
    return data;
}

module.exports = { getAgents, getAgent };
