const {
    CLOUD_MANAGER_AGGREGATES_PATH_PREFIX, OCCM_BASE_URL, CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX
} = require('../../utils/consts');
const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const { aggregatesResponseSchema, objectStoreAttachRequestSchema } = require('../../joi-schemas/cloud-manager-schamas');
const getLogger = require('../../utils/logger/logger');
const { getRequest, postRequest } = require('./cloud-manager-axios-utils');

const logger = getLogger();


async function getAggregates(clusterId) {
    logger.info(`Get aggregate of cluster ${clusterId}`);
    const data = await getRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`);
    validateResponse(data, aggregatesResponseSchema);
    return data;
}

async function attachObjectStore(clusterId, aggregateName, checkOnly, objectStoreName, allowFlexgroup = false) {
    logger.info(`attach object store for cluster ${clusterId} and aggregate ${aggregateName} and checkOnly ${checkOnly} and allowFlexgroup ${allowFlexgroup}`);
    const requestData = {
        objectStoreName,
        aggregateName,
        checkOnly,
        workingEnvironmentId: clusterId,
        allowFlexgroup: (allowFlexgroup) || undefined
    };
    validateRequest(requestData, objectStoreAttachRequestSchema);
    return postRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, requestData);
}

module.exports = { getAggregates, attachObjectStore };
