const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const { volumesResponseSchema, modifyVolumeRequestSchema } = require('../../joi-schemas/cloud-manager-schamas');
const {
    CLOUD_MANAGER_VOLUMES_PATH_PREFIX, OCCM_BASE_URL
} = require('../../utils/consts');
const getLogger = require('../../utils/logger/logger');
const { getRequest, putRequest } = require('./cloud-manager-axios-utils');


const logger = getLogger();


async function getVolumes(clusterId) {
    logger.info(`get volumes of cluster ${clusterId}`);
    const data = await getRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${clusterId}`);
    validateResponse(data, volumesResponseSchema);
    return data;
}

async function modifyVolume(clusterId, svmName, volumeName, tieringPolicy) {
    const requestData = { tieringPolicy };
    logger.info(`modifying volume tiering policy ${volumeName} of clusterId ${clusterId} and svmName ${svmName} to ${tieringPolicy}`);
    validateRequest(requestData, modifyVolumeRequestSchema);
    return putRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${clusterId}/${svmName}/${volumeName}`, requestData);
}


module.exports = { getVolumes, modifyVolume };
