/*
const bytes = require('bytes');
const {
    intersection, get, flatten
} = require('lodash');
const Account = require('../models/account');
const { getAllClustersInAccountInternal } = require('./multi-tenancy-operation');
const { listAllSubscriptionsForService } = require('../lib/subscription-service');
const { UNSUBSCRIBE_PENDING, SUBSCRIBED, PENDING_SUBSCRIPTION } = require('../utils/consts');
const charge = require('../lib/charging-service');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

async function findAccountsWithoutFreeTrial() {
    return Account.find({ freeTrialExpiryDate: { $lt: new Date() } }).exec();
}

// subscribedAccount - an array of arrays of accountIds
// candidatesAccounts an array of accountsIds
function findAccountsToCharge(candidatesAccounts, subscribedAccounts) {
    console.log('Subscribed Accounts', subscribedAccounts);
    const uniqueSubscribedAccounts = [...new Set(flatten(subscribedAccounts))]; //delete the set
    return intersection(uniqueSubscribedAccounts, candidatesAccounts);
}

async function findTieringUsagePerAccount(accountId) {
    const clusters = await getAllClustersInAccountInternal(accountId);
    return clusters.map(cluster => {
        const tieredCapacity = get(cluster, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace');
        const clusterUuid = get(cluster, 'ontapClusterProperties.clusterUuid'); // TODO - do we want clusterId or clusterUuid??
        if (tieredCapacity) {
            const { size, unit } = tieredCapacity;
            // val should be in GB
            return { resourceId: clusterUuid, val: bytes(`${size}${unit}`) / (1024 ** 3) };
        }
        return { resourceId: clusterUuid, val: 0 };
    });
}

async function createAccountMap(accountsFromSubscriptions) {
    logger.info('create accont Map - with accountId and resources for each account');
    const candidatesAccounts = await findAccountsWithoutFreeTrial();
    const candidatesAccountIds = candidatesAccounts.map(account => account.accountId);
    // find accounts to charge findAccountsToCharge
    const accountsToCharge = findAccountsToCharge(candidatesAccountIds, accountsFromSubscriptions);
    // reduce create accounts Map - accountId: {accountId, resources}
    const accounts = await Promise.all(accountsToCharge.map(async accountId => {
        const resourcesForAccount = await findTieringUsagePerAccount(accountId);
        return [accountId, resourcesForAccount];
    }));

    return new Map([accounts]);
}

function createObjectToSend(subscriptionId, accounts, timestamp) {
    return {
        subscriptionId, accounts, timestamp, dimension: 'Tier1'
    };
}

function createSubscriptionToAccountObject(subscriptionsArray) {
    // return subscriptionsArray.reduce((accumulator, currentVal) => ({ ...accumulator, [currentVal.subscriptionId]: currentVal.accounts }), {});
    return subscriptionsArray.reduce((accumulator, currentVal) => {
        logger.info('Inside subscriptionArray reduce', JSON.stringify(currentVal), JSON.stringify(accumulator));
        const { subscriptionId, accounts } = currentVal;
        if (accumulator[subscriptionId]) {
            logger.error('There is already subscription with this subscriptionId', subscriptionId);
            accumulator[subscriptionId].push(...accounts);
        }
        accumulator[subscriptionId] = accounts;
        return accumulator;
    }, {});
}

async function collectChargingInfo() {
    const timestamp = new Date().toISOString();

    // get all subscription per service
    const allSubscriptionsForTiering = await listAllSubscriptionsForService();
    // filter all subscription that can be charged
    const allValidSubscriptions = allSubscriptionsForTiering.filter(subscription => [UNSUBSCRIBE_PENDING, SUBSCRIBED, PENDING_SUBSCRIPTION].includes(subscription.subsctiptionStatus));


    const subscriptionObject = createSubscriptionToAccountObject(allValidSubscriptions);
    // does it relevant to do an object from subscriptionId to accounts?

    const accountsFromSubscriptions = Object.values(subscriptionObject);
    const accountMap = await createAccountMap(accountsFromSubscriptions);


    // add account resources to subscription objects
    Object.keys(subscriptionObject).forEach(subscriptionId => {
        const accounts = subscriptionObject[subscriptionId];
        const accountsWithResources = accounts.map(account => {
            if (accountMap.has(account)) {
                const resources = accountMap.get(account);
                accountMap.delete(account);
                return { accountId: account, resources };
            }
            return undefined;
        });
        subscriptionObject[subscriptionId] = compact(accountsWithResources);
    });

    // create the array that needs to be sent
    const chargingToSend = Object.keys(subscriptionObject).map(subscriptionId => createObjectToSend(subscriptionId, subscriptionObject[subscriptionId], timestamp));
    return charge(chargingToSend);
}


module.exports = {
    findAccountsWithoutFreeTrial, findTieringUsagePerAccount, collectChargingInfo, findAccountsToCharge, createSubscriptionToAccountObject
};

*/
