// const config = require('config');
const getLogger = require('../utils/logger/logger');
const { getAllClustersInAccountInternal } = require('../operations/multi-tenancy-operation');
const { getVolumes, modifyVolume } = require('../lib/cloud-manager/volumes');

const logger = getLogger();


// #1 find all bilable accounts
// #2 for each account getworkspace
// #3 for each workspace clustersInfo
// #4 sum all the usedCapacity( or other field )
// #5 create metering entry
// #6 send to AWS
// #7 insert to the saasBilling collectionand send again the failed ones


async function createAccountAndClusterObject(accountId) {
    const clusters = await getAllClustersInAccountInternal(accountId);
    return { accountId, clusters };
}

async function getAllVolumesOfAccount(serviceToken, workSpaceObject) {
    const { clustersIds } = workSpaceObject;
    return Promise.all(clustersIds.map(async clusterId => {
        const volumes = await getVolumes(serviceToken, clusterId);
        return { clusterId, volumes };
    }));
}

async function stopTiering(serviceToken, volumeAndCluster) {
    const { clusterId, volumes } = volumeAndCluster;
    const promises = volumes.map(async volume => {
        const { svmName, name, tieringPolicy } = volume;
        if (tieringPolicy === 'auto') {
            await modifyVolume(serviceToken, clusterId, svmName, name, 'none');
        }
    });
    await Promise.all(promises);
}

async function stopTieringVolumesInAccount(serviceToken, accountId) {
    logger.info(`handeling unsubscribe of volumes in account ${accountId}`);
    // #2 find all clusters - {accountId, [clustersId]}
    const accountsWithClusters = await createAccountAndClusterObject(serviceToken, accountId);
    // #3 find all volumes of workspace = [{clusterId, volumes}]
    const clusterWithVolumes = await Promise.all(accountsWithClusters.map(accountObject => getAllVolumesOfAccount(serviceToken, accountObject)));
    // #4 change tier policy
    await Promise.all(clusterWithVolumes.map(volumeObject => stopTiering(serviceToken, volumeObject)));
}


module.exports = {
    createAccountAndClusterObject,
    stopTieringVolumesInAccount
};
