const { first, sortBy } = require('lodash');
const moment = require('moment');
const Account = require('../models/account');
const { listSubscriptionsOfAccount } = require('../lib/subscription-service');
const {
    SUBSCRIBE_SUCCESS, CLOUD_TIERING_NAMESPACE,
    TIERING_SERVICE_NAME_SUBCRIPTION, UNSUBSCRIBED, UNSUBSCRIBE_PENDING
} = require('../utils/consts');


const getLogger = require('../utils/logger/logger');

const logger = getLogger();


async function calculateDaysLeftForTrial(accountId) {
    const account = await Account.findOne({ accountId });
    const { freeTrialExpiryDate } = account;
    const daysLeftForTrial = moment(freeTrialExpiryDate).startOf('day').diff(moment().startOf('day'), 'days');
    return (daysLeftForTrial >= 0) ? daysLeftForTrial : -1;
}

async function findSubscription(accountId) {
    logger.info('Find subscription status for account', accountId);
    const subscriptionList = await listSubscriptionsOfAccount(accountId);
    logger.info('subscriptionList', subscriptionList);
    return subscriptionList.find(subscription => subscription.serviceName === TIERING_SERVICE_NAME_SUBCRIPTION && subscription.subscriptionStatus === SUBSCRIBE_SUCCESS);
}

async function findSubscriptionStatus(accountId) {
    const subscriptionsList = await listSubscriptionsOfAccount(accountId);
    // TODO add filter by service name - once jonathan wiil support service name
    if (subscriptionsList.length) {
        const subscription = first(sortBy(subscriptionsList, 'createdAt'));
        return subscription.subscriptionStatus === UNSUBSCRIBE_PENDING ? UNSUBSCRIBE_PENDING.replace('-', '_') : subscription.subscriptionStatus;
    }
    return UNSUBSCRIBED;
}

async function findAccountInfo() {
    const accountId = CLOUD_TIERING_NAMESPACE.get('accountId');
    const daysLeftForTrial = await calculateDaysLeftForTrial(accountId);
    const subscriptionStatus = await findSubscriptionStatus(accountId);
    return { accountId, daysLeftForTrial, subscriptionStatus };
}

module.exports = {
    calculateDaysLeftForTrial,
    findSubscription,
    findSubscriptionStatus,
    findAccountInfo
};
