const mongoose = require('mongoose');
const moment = require('moment');
const Account = require('../../src/models/account');
const getLogger = require('../../src/utils/logger/logger');
const { DB_CONNECTION_STRING } = require('../../src/utils/consts');

const logger = getLogger();
const accountId = '1';
const name = 'test';

beforeAll(async () => {
    try {
        await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
    } catch (e) {
        logger.error('Cannot connect to mongoDB');
    }
});

afterAll(async done => {
    try {
        await mongoose.connection.close();
        done();
    } catch (e) {
        logger.error('Error closing connection', e);
    }
});

describe('Account Model -CRUD', () => {
    beforeEach(() => Account.deleteMany({}));

    test('create', async () => {
        const accountDocument = await Account.create({ name, accountId });
        expect(accountDocument.accountId).toEqual(accountId);
        const { freeTrialExpiryDate } = accountDocument;
        const expectedFreeTrailExpiryDate = new Date(moment().add(60, 'days'));
        // verify daty
        expect(freeTrialExpiryDate.getDay()).toEqual(expectedFreeTrailExpiryDate.getDay());

        // verify year
        expect(freeTrialExpiryDate.getMonth()).toEqual(expectedFreeTrailExpiryDate.getMonth());

        // verify month
        expect(freeTrialExpiryDate.getFullYear()).toEqual(expectedFreeTrailExpiryDate.getFullYear());
    });

    test('get', async () => {
        await Account.create({ name, accountId: '2' });
        const foundDoc = await Account.findOne({ accountId: '2' });
        expect(foundDoc.accountId).toEqual('2');
    });

    test('update', async () => {
        await Account.create({ name, accountId });
        const freeTrialExpiryDate = moment().add(3, 'days');
        await Account.update({ accountId }, { $set: { freeTrialExpiryDate } });
        const updatedAccount = Account.findOne({ freeTrialExpiryDate });
        expect(updatedAccount).toBeTruthy();
    });

    test('delete', async () => {
        const doc = await Account.create({ name, accountId });
        await doc.remove();
        expect(doc.$isDeleted()).toEqual(true);
    });
});

describe('Account Model -Custom Methods', () => {
    test('toJson', async () => {
        await Account.create({ name, accountId });
        const result = await Account.findOne({ accountId });
        expect(result.accountId).toEqual(accountId);
    });
});
