const { URL } = require('url');
const nock = require('nock');
const { last, cloneDeep } = require('lodash');
const { validateSchema } = require('../../../src/utils/schema-validation');
const { subscriptionSchema } = require('../../../src/joi-schemas/subscription-schemas');
const {
    SUBSCRIPTION_ENDPOINT
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { STATE, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const subscriptionServiceScope = nock(SUBSCRIPTION_ENDPOINT)
    .log(evt => logger.info(evt))
    // create subscription - fake API for testing
    .post(escapeForwardSlash('/.+'),
        body => {
            try {
                validateSchema(body, subscriptionSchema);
                return true;
            } catch (err) {
                return false;
            }
        })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const userId = findUserId(this.req.headers.authorization);
        const requestURL = new URL(uri, SUBSCRIPTION_ENDPOINT);
        const accountId = last(requestURL.pathname.split('/'));
        const subscription = JSON.parse(requestBody);
        const { subscriptionId, subscriptionStatus } = subscription;
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        if (USERS[userId].SUBSCRIPTIONS[subscriptionId]) {
            const { accounts } = USERS[userId].SUBSCRIPTIONS[subscriptionId];
            if (!accounts.find(account => account === accountId)) {
                USERS[userId].SUBSCRIPTIONS[subscriptionId].accounts.push(accountId);
            }
            USERS[userId].SUBSCRIPTIONS[subscriptionId].subscriptionStatus = subscriptionStatus;
        } else {
            subscription.accounts = [accountId];
            USERS[userId].SUBSCRIPTIONS[subscriptionId] = subscription;
        }
        return [200, subscription];
    })

    // list subscriptions
    .get(escapeForwardSlash('/.+'))
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, SUBSCRIPTION_ENDPOINT);
        const accountId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers['x-user-token']);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        if (USERS[userId].ACCOUNTS[accountId]) {
            const subscriptions = Object.values(USERS[userId].SUBSCRIPTIONS).filter(subscription => subscription.accounts.includes(accountId));
            return [200, subscriptions];
        }
        return [404, 'Account was not found'];
    });

subscriptionServiceScope.persist(true);
