const { readFileSync } = require('fs');
const { join } = require('path');
const { cloneDeep } = require('lodash');
const getLogger = require('../../src/utils/logger/logger');

const logger = getLogger('simulator');

const STATE = {
    ACCOUNTS: {},
    AGENTS: {},
    AGGREGATES: {},
    VOLUMES: {},
    CLUSTERS: {},
    WORKSPACES: {},
    SUBSCRIPTIONS: {}
};

const USERS = {};

setInterval(() => logger.info('STATE:', JSON.stringify(STATE, null, 4), '\n USERS_STATE', JSON.stringify(USERS, null, 4)), 30 * 1000);

function reset(key, userId) {
    if (key) {
        if (userId) {
            logger.info('DELETING key', key, 'For user', userId);
            Object.keys(USERS[userId][key]).forEach(subKey => delete USERS[userId][key][subKey]);
        } else {
            logger.info('DELETING key', key, 'For all users');
            Object.keys(STATE[key]).forEach(subKey => delete STATE[key][subKey]);
            Object.keys(USERS).forEach(user => {
                USERS[user] = cloneDeep(STATE);
            });
        }
    } else if (userId) {
        logger.info('DELETING user data from USERS_STATE');
        USERS[userId] = {};
    } else {
        logger.info('DELETING all data for all users');
        Object.keys(STATE).forEach(subKey => {
            Object.keys(STATE[subKey]).forEach(entity => delete STATE[subKey][entity]);
        });
        Object.keys(USERS).forEach(user => delete USERS[user]);
    }
}


function loadScenrioForUser(scenarioName, userId) {
    logger.info('Loading scenario', scenarioName, 'for user', userId);
    const SCENARIOS_DIRECTORY = 'scenarios';

    try {
        const scenario = readFileSync(join(__dirname, SCENARIOS_DIRECTORY, `${scenarioName || 'default'}.json`));
        reset(undefined, userId);
        USERS[userId] = cloneDeep(JSON.parse(scenario));
    } catch (err) {
        logger.error('Failed to load scenario', err);
    }
}

// reset user to default state - using by simulator
function resetStateForUser(userId) {
    logger.info(`reset state for userId ${userId}`);
    USERS[userId] = cloneDeep(STATE);
}

function loadScenario(scenarioName) {
    logger.info('Loading scenario', scenarioName);
    const SCENARIOS_DIRECTORY = 'scenarios';

    try {
        const scenario = readFileSync(join(__dirname, SCENARIOS_DIRECTORY, `${scenarioName || 'default'}.json`));
        reset();
        Object.assign(STATE, JSON.parse(scenario));
    } catch (err) {
        logger.error('Failed to load scenario', err);
    }
}


module.exports = {
    STATE, USERS, loadScenario, reset, loadScenrioForUser, resetStateForUser
};
